# if defined(UNDER_CE)
#  include <winbase.h>
# else
#  include <winver.h>
# endif

VS_VERSION_INFO VERSIONINFO
	FILEVERSION 4,6,1,0
	PRODUCTVERSION 4,6,1,0
	FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
	FILEFLAGS VS_FF_DEBUG
#else
	FILEFLAGS 0x0L
#endif
	FILEOS VOS__WINDOWS32
	FILETYPE VFT_DLL
	FILESUBTYPE 0x0L
	BEGIN
		BLOCK "StringFileInfo"
		BEGIN
			BLOCK "040904B0"
			BEGIN
				VALUE "CompanyName", "Nokia Corporation and/or its subsidiary(-ies)\0"
				VALUE "FileDescription", "C++ application development framework.\0"
				VALUE "FileVersion", "4.6.1.0\0"
				VALUE "LegalCopyright", "Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies)\0"
				VALUE "OriginalFilename", "qsqlite4.dll\0"
				VALUE "ProductName", "Qt4\0"
			END
		END
	END
/* End of Version info */

